﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.IO;
using System.IO.IsolatedStorage;

namespace FileSaveSample
{
    public partial class MainPage : PhoneApplicationPage
    {
        // Constructor
        public MainPage()
        {
            InitializeComponent();
        }

        private void downloadButton_Click(object sender, RoutedEventArgs e)
        {
            string fileUrl = "http://localhost:2975/Download"; // 파일 다운로드를 위한 URL 경로

            // 웹사이트에서 파일을 다운로드 하기 위한 Request를 생성한다.
            // Silverlight 에서는 WebRequest에 대한 동기 호출인 GetResponse 메서드를 지원하지 않으며,
            // 비동기 호출인 BeginGetResponse 메서드를 이용하여야 한다.
            
            HttpWebRequest request = HttpWebRequest.Create(fileUrl) as HttpWebRequest;
            request.BeginGetResponse(new AsyncCallback(DownloadCallback), request);
        }

        private void DownloadCallback(IAsyncResult asyncronousResult)
        {
            // BeginGetResponse 메서드의 state 매개변수로부터 전달받은 HttpWebRequest객체의
            // EndGetResponse 메서드를 호출하여 HttpWebResponse 객체를 획득한다.
            HttpWebRequest request = (HttpWebRequest)asyncronousResult.AsyncState;  
            HttpWebResponse response = (HttpWebResponse)request.EndGetResponse(asyncronousResult);

            // 저장할 파일명을 가져옵니다.
            string filename = GetFileName(response.Headers["Content-Disposition"]);

            try
            {
                // HttpWebResponse의 스트림을 이용하여 격리된 저장소에 파일을 쓴다.
                using (Stream stream = response.GetResponseStream())
                {
                    IsolatedStorageFile appStorage = IsolatedStorageFile.GetUserStoreForApplication();
                    using (IsolatedStorageFileStream fileStream = new IsolatedStorageFileStream(filename, FileMode.OpenOrCreate, appStorage))
                    {
                        byte[] buffer = new byte[32768];
                        int readBytes;

                        while ((readBytes = stream.Read(buffer, 0, buffer.Length)) > 0)
                        {
                            fileStream.Write(buffer, 0, readBytes);
                        }
                    }
                }
            }
            catch (Exception) { }

            // UI Thread에 접근하려고 할 경우에는 UnauthorizedAccessException이 발생한다.
            this.Dispatcher.BeginInvoke(delegate()
            {
                DisplayFiles();
            });
        }

        private string GetFileName(string httpHeader)
        {
            System.Text.RegularExpressions.Regex regex = new System.Text.RegularExpressions.Regex("filename=\"?(?<filename>[^\"]+)\"?");
            return regex.Match(httpHeader).Groups["filename"].Value;
        }

        private void DisplayFiles() 
        {
            // 격리된 저장소에 저장된 전체 파일들의 이름을 ListBox에 출력한다.
            IsolatedStorageFile appStorage = IsolatedStorageFile.GetUserStoreForApplication();

            fileListBox.Items.Clear();

            string[] files = appStorage.GetFileNames();

            foreach (string filename in files)
            {
                fileListBox.Items.Add(filename);
            }
        }

        private void fileListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (fileListBox.SelectedItem != null)
            {
                // 선택된 파일을 읽어서 출력한다.
                string selectedFilename = fileListBox.SelectedItem.ToString();

                if (System.IO.Path.GetExtension(selectedFilename) == ".txt")
                {
                    IsolatedStorageFile appStorage = IsolatedStorageFile.GetUserStoreForApplication();

                    using (StreamReader reader = new StreamReader(new IsolatedStorageFileStream(selectedFilename, FileMode.Open, appStorage)))
                    {
                        string content = reader.ReadToEnd();

                        contentTextBlock.Text = content;
                    }
                }
            }
        }
    }
}
